import numpy as np
from cython.parallel import prange
cimport cython

@cython.boundscheck(False)
def julia_iter(int maxiter, double c_real, double c_imag,
               double[:] x, double[:] y, int[:] output):
    """
    Julia Updating Rule
    """
    cdef int i
    cdef double z_real, z_imag, tmp_real, tmp_imag
    cdef int num_x = len(x)
    
    with nogil:    
        for i in prange(num_x, schedule=guided, num_threads=2): 
        
            z_real = x[i]
            z_imag = y[i]
            tmp_real = z_real**2
            tmp_imag = z_imag**2
            output[i] = 0          
    
            while output[i] < maxiter and tmp_real + tmp_imag < 4:
                z_imag = 2*z_imag*z_real + c_imag
                z_real = tmp_real - tmp_imag + c_real                
                tmp_real = z_real**2
                tmp_imag = z_imag**2    
                output[i] += 1
                
    #return output