# imports #
import time
import os
import numpy as np
from julia_iter import *
from cffi import FFI

clear = lambda: os.system('cls')


# cython #
os.system("cython -a cython_fn.pyx")
os.system("python setup.py build_ext --inplace")

os.system("cython -a cython_fn_openmp.pyx")
os.system("python setup_openmp.py build_ext --inplace ")

clear()

import cython_fn
import cython_fn_openmp


# cffi #
ffi = FFI()
os.system("gcc -fopenmp -o3 -std=gnu99 -c julia_iter_c.c -o julia_iter_c.o")
os.system("gcc -fopenmp -shared -o julia_iter_c.so julia_iter_c.o")
    
    
ffi.cdef(r''' void julia_iter(int maxiter, double c_real, double c_imag,
                              double *x, double*y, int *output,
                              int num_x); ''')
                             
lib = ffi.dlopen("julia_iter_c.so")  
    
    
# area of complex space to investigate #
x1, x2, y1, y2 = -1.8, 1.8, -1.8, 1.8
c_real, c_imag = -0.62772, -.42193


# functions #       
def julia_iter_wrapper(method, n, max_iter):
        
    # create grids #        
    x = np.linspace(x1, x2, n)
    y = np.linspace(y1, y2, n)
    x_grid, y_grid = np.meshgrid(x, y)
    
    x_grid = np.ravel(x_grid)
    y_grid = np.ravel(y_grid)
            
    # allocate memory #
    output = np.empty(len(x_grid), dtype=np.int)
    
    # do calculations #
    start_time = time.time()

    if method == 'standard':
        
        output = julia_iter(max_iter, c_real, c_imag, x_grid, y_grid, output)        

    elif method == 'cython':
        
        cython_fn.julia_iter(max_iter, c_real, c_imag, x_grid, y_grid, output)

    elif method == 'cython_openmp':
        
        cython_fn_openmp.julia_iter(max_iter, c_real, c_imag, x_grid, y_grid, output)   
    
    elif method == 'cffi_openmp':
        
        p_x_grid = ffi.cast('double *', x_grid.ctypes.data)
            
        p_y_grid = ffi.cast('double *', y_grid.ctypes.data)
        p_output = ffi.cast('int *', output.ctypes.data)        
        
        num_x = len(x_grid);
        
        lib.julia_iter(max_iter, c_real, c_imag, 
                       p_x_grid, p_y_grid, p_output,
                       num_x)                       
        a = 2
                       
    
    end_time = time.time()
    secs = end_time - start_time
    print "Took", secs, "seconds"

    validation_sum = sum(output)
    print "Validation sum:", validation_sum

# Run      
if __name__ == "__main__":
           
    print("\nStandard:")
    julia_iter_wrapper(method='standard', n=100, max_iter=300)

    print("\nCython:")
    julia_iter_wrapper(method='cython', n=100, max_iter=300)
    
    print("\nCython + OpenMp:")
    julia_iter_wrapper(method='cython_openmp', n=100, max_iter=300)    
    
    print("\nCython:")
    julia_iter_wrapper(method='cython', n=1000, max_iter=5000)
    
    print("\nCython + OpenMp:")
    julia_iter_wrapper(method='cython_openmp', n=1000, max_iter=5000)        
    
    print("\nCFFI:")
    julia_iter_wrapper(method='cffi_openmp', n=1000, max_iter=5000)  